'''
游戏引擎：控制游戏流程
'''
import game_sprites
import pygame
import time

# 自定义一个事件
ENEMY_CREATE = pygame.USEREVENT
pygame.time.set_timer(ENEMY_CREATE, 1000)


class GameEngine:
    """游戏引擎类型"""

    def __init__(self):
        # 初始化所有模块
        pygame.init()

        # 定义游戏窗口
        self.screen = pygame.display.set_mode(game_sprites.SCREEN_SIZE)

        # 定义背景和英雄飞机精灵组对象
        self.resources = pygame.sprite.Group()

        # 定义敌人飞机精灵组对象
        self.enemys = pygame.sprite.Group()

        # 定义时钟对象
        self.clock = pygame.time.Clock()

        # 音乐精灵组
        self.music = game_sprites.Music()

        # 敌机爆炸精灵组
        self.bombs = pygame.sprite.Group()

        # 定义份数
        self.num = 0

    def fen_shu(self):

        self.font = pygame.font.SysFont("宋体", 50, True, True)
        self.score = self.font.render("score: %s" % self.num, True, (255, 0, 255))

    def create_scene(self):
        """创建游戏背景、英雄飞机
            把游戏中的精灵添加到精灵组中
        """
        # self.resources.empty()
        bg1 = game_sprites.BackgroundSprite("./images/bg_img_1.jpg", 1)
        bg2 = game_sprites.BackgroundSprite("./images/bg_img_1.jpg", 1, perpare = True)
        self.hero = game_sprites.HeroSprite("./images/hero_1.png", speed=0)
        self.resources.add(bg1, bg2, self.hero)
        # 将精灵对象添加到精灵组对象中

    def update_scene(self):
        """渲染场景:把精灵组渲染到屏幕中"""

        # 背景和英雄飞机精灵组更新渲染
        self.resources.update()
        self.resources.draw(self.screen)
        self.screen.blit(self.score, (10, 10))

        # 英雄飞机子弹精灵组更新渲染
        self.hero.bullets.update()
        self.hero.bullets.draw(self.screen)

        # 敌人飞机精灵组更新渲染
        self.enemys.update()
        self.enemys.draw(self.screen)

    def check_collide(self):
        """碰撞检测"""

        # 两种碰撞方式:1.子弹和敌方飞机
        self.bombs.add(pygame.sprite.groupcollide(self.enemys, self.hero.bullets, True, True))
        for enemy_bomb in self.bombs:
            self.num += 2
            print(self.num)
            self.bomb(enemy_bomb)
            self.music.bomb_music()
            self.bombs.remove(enemy_bomb)

        # 2.英雄飞机和敌方飞机之间
        e = pygame.sprite.spritecollide(self.hero, self.enemys, True)
        if len(e) > 0:
            self.num = 0
            self.music.bomb_music()
            time.sleep(1)
            self.hero.kill()
            self.game_over()

    def show_index(self):
        """展示首页"""

        # 首页精灵
        self.index = game_sprites.Index("./images/shou_ye_1.jpg")
        self.resources.add(self.index)

        while True:
            self.resources.draw(self.screen)
            self.resources.update()
            pygame.display.update()
            # 键盘事件监听
            key_down = pygame.key.get_pressed()
            event_list = pygame.event.get()
            if len(event_list) > 0:
                # print(event_list)

                for event in event_list:
                    # 如果当前事件为QUIT事件
                    if event.type == pygame.QUIT:
                        # 卸载所有资源，退出程序
                        pygame.quit()
                        exit()
            if key_down[pygame.K_a]:
                print("游戏开始...")
                self.resources.empty()
                return self.start()

    def game_over(self):
        """结束界面"""
        game_over_bg = pygame.image.load("./images/game_over_bg.jpg")
        self.screen.blit(game_over_bg, (0, 0))
        self.screen.blit(self.score, (160, 200))
        pygame.display.update()

        while True:
            for event in pygame.event.get():
                if event.type == pygame.MOUSEBUTTONDOWN:
                    if 150 <= event.pos[0] <= 400 and 450 <= event.pos[1] <= 550:
                        self.resources.empty()
                        self.enemys.empty()
                        self.start()
                    elif 150 <= event.pos[0] <= 400 and 550 <= event.pos[1] <= 650:
                        pygame.quit()
                        exit()
                elif event.type == pygame.QUIT:
                    pygame.quit()
                    exit()

    def check_event(self):
        """监听所有事件"""

        # 隔一段时间创建一家敌方飞机
        event_list = pygame.event.get()
        if len(event_list) > 0:
            for event in event_list:
                # 如果当前事件为QUIT事件
                if event.type == pygame.QUIT:
                    # 卸载所有资源，退出程序
                    pygame.quit()
                    exit()
                if event.type == ENEMY_CREATE:
                    print("创建一家敌方飞机...")
                    enemy = game_sprites.EnemySprite()
                    # 添加到敌方飞机精灵组中
                    self.enemys.add(enemy)

        # 键盘监听
        key_down = pygame.key.get_pressed()

        if key_down[pygame.K_LEFT]:
            print("飞机向左移动<<<")
            self.hero.rect.x -= 5
        if key_down[pygame.K_RIGHT]:
            print("飞机向右移动>>>")
            self.hero.rect.x += 5
        if key_down[pygame.K_UP]:
            print("飞机向上移动^^^")
            self.hero.rect.y -= 5
        if key_down[pygame.K_DOWN]:
            print("飞机向下移动vvv")
            self.hero.rect.y += 5
        if key_down[pygame.K_SPACE]:
            self.hero.fire()
            self.music.bullet_music()
            print("子弹发射",self.hero.bullets)

    def bomb(self, enemy):
        """子弹爆炸效果"""
        for img_path in ["./images/bomb1.png", "./images/bomb2.png", "./images/bomb3.png"]:
            image = pygame.image.load(img_path)
            self.screen.blit(image, (enemy.rect.x + 20, enemy.rect.y))
            pygame.display.update()
            # self.bombs.draw(self.screen)

    def start(self):
        print("游戏开始...")
        # 刷新游戏帧率
        self.clock.tick(24)
        self.create_scene()
        self.music.back_music()

        while True:

            # 调用分数方法
            self.fen_shu()
            # 更新绘制、渲染精灵组
            self.update_scene()
            # 事件监听
            self.check_event()
            # 碰撞检测
            self.check_collide()
            # 游戏更新展示
            pygame.display.update()




'''
存放飞机大战中所有精灵类型的文件
author:jiaojiao
time:2018/8/5
'''
import pygame,random

# 定义需要的常量
SCREEN_SIZE = (500, 700)
SCREEN_RECT = pygame.Rect(0, 0, *SCREEN_SIZE)


# 定义游戏精灵类型
class GameSprite(pygame.sprite.Sprite):
    """游戏精灵对象，用于表示游戏中的各个元素"""

    def __init__(self,image_path,speed=0):
        # 初始化数据
        super().__init__()
        self.image = pygame.image.load(image_path)
        self.rect = self.image.get_rect()
        self.speed = speed

    def update(self):
        # 默认运动更新方式
        self.rect.y += self.speed


# 定义游戏背景类型
class BackgroundSprite(GameSprite):

    def __init__(self, image_path, speed, perpare=False):
        # 继承父类
        super().__init__(image_path, speed)

        if perpare:
            self.rect.y = -SCREEN_SIZE[1]

    def update(self):
        # 调用父类方法，执行运动
        super().update()
        # 判断背景图片
        if self.rect.y > SCREEN_SIZE[1]:
            self.rect.y = -SCREEN_SIZE[1]


# 定义英雄飞机类型
class HeroSprite(GameSprite):
    """英雄飞机精灵对象"""

    def __init__(self, image_path, speed):
        # 初始化英雄飞机的图片，速度
        super().__init__(image_path, speed=0)
        self.speed = speed
        # 初始化英雄飞机的位置
        self.rect.centerx = SCREEN_RECT.centerx
        self.rect.centery = SCREEN_RECT.centery + 200
        # 子弹精灵组
        self.bullets = pygame.sprite.Group()

    def update(self):
        # 水平边界判断
        if self.rect.x <= 0:
            self.rect.x = 0
        elif self.rect.x >= SCREEN_RECT.width - self.rect.width:
            self.rect.x = SCREEN_RECT.width - self.rect.width


        # 垂直边界判断
        if self.rect.y <= 0:
            self.rect.y = 0
        elif self.rect.y > SCREEN_RECT.height - self.rect.height:
            self.rect.y = SCREEN_RECT.height - self.rect.height

    def fire(self):
        """英雄飞机发射子弹"""
        # 创建一个子弹对象
        if len(self.bullets) < 7:
            bullet = BulletSprite(self.rect.centerx, self.rect.y)
            # 将子弹加到精灵组对象
            self.bullets.add(bullet)


class BulletSprite(GameSprite):
    """子弹精灵"""
    def __init__(self, x, y):
        # 获取子弹图片
        super().__init__("./images/hero_bullet.png", speed=-4)
        self.rect.x = x - 20
        self.rect.y = y

    def update(self):
        # 调用父类方法进行操作
        super().update()
        # 边界判断
        if self.rect.y <= -self.rect.height:
            self.kill()

    def __del__(self):
        print("子弹对象销毁")


class EnemySprite(GameSprite):
    """敌机精灵对象"""

    def __init__(self):
        # 初始化敌机数据：图片，速度
        super().__init__("./images/enemy.png", speed=random.randint(2, 3))
        # 初始化敌方飞机位置
        self.rect.x = random.randint(0, SCREEN_RECT.width - self.rect.width)
        self.rect.y = -self.rect.height

    def update(self):
        # 调用父类方法，执行运动
        super().update()
        # 子类中判断边界
        if self.rect.y > SCREEN_RECT.height:
            # 飞机一旦超出屏幕就销毁
            self.kill()

    def __del__(self):
        self.destroy()

    def destroy(self):
        print("敌方飞机销毁")


class Music:
    """游戏中音乐，音效"""

    def back_music(self):
        music1 = pygame.mixer.Sound("./music/game_music.wav")
        music1.play()

    def bullet_music(self):
        music2 = pygame.mixer.Sound("./music/bullet.wav")
        music2.play()

    def bomb_music(self):
        music3 = pygame.mixer.Sound("./music/enemy_down.wav")
        music3.play()


class Index(GameSprite):
    """定义首页类"""

    def __init__(self, image_path):
        # 继承父类
        super().__init__(image_path, speed=0)

    def update(self):

        super().update()


class EnemyBulletSprite(GameSprite):
    """敌机子弹精灵"""

    def __init__(self, x, y):
        # 获取子弹图片
        super().__init__("./images/enemy_bullet.png", speed=4)



# 引入构建包信息模块
from distutils.core import setup

# 定义发布的包文件的信息
setup(
    name="fight_plane",
    version="1.0",
    description="娇娇的飞机大战",
    author="淡陌",
    author_email="2293693518@qq.com",
    py_modules=['__init__', 'game_engine', 'game_sprites', 'main']
)
